import React from 'react'
import _ from 'lodash'
import PropTypes from 'prop-types'
import { displayTextWithLink, displayTextWithLinkWithHighlightWithFocus } from '../utils'
import { detectPartOfLink, getIndicesOfLink, getLinkUrl, orderChatIndex, initialList } from '../utils/text'
import '../index.scss'

const HighlightContain = ({ text, keyword, currentIndex }) => {

  console.log(currentIndex)
  const hasPartOfLink = detectPartOfLink(text)
  let linkIndices = initialList(text.length, 0)
  let linkName = initialList(text.length, 0)
  if (hasPartOfLink) {
    linkIndices = getIndicesOfLink(text, linkIndices)
    linkName = getLinkUrl(text, linkIndices)
  }
  // start display along as match and index logic
  const isKeywordHasAllSpace = keyword.trim()
  if (isKeywordHasAllSpace.length === 0) {  // no keyword
    if (!hasPartOfLink) {
      return (<span className="text-highlight-no-modify">{text}</span>)
    } else if (hasPartOfLink) {
      return displayTextWithLink(text, linkIndices, linkName)
    }
  }
  // else is that it has keyword
  const modifiedText = text.toLowerCase()
  const trimedKeyword = keyword.toLowerCase().trim()
  const splitedKeywordList = trimedKeyword.split(new RegExp(/\s+/, 'gi'))
  let modifiedKeyword = _.uniq(splitedKeywordList)
  // start logic match with index
  let currentIndicesMatches = [initialList(modifiedText.length, 0), []]
  modifiedKeyword.forEach((word) => {
    if (word) {
      currentIndicesMatches = getIndicesOfMatchesContain(currentIndicesMatches[0], word, modifiedText, currentIndicesMatches[1])
    }
  })
  const listOfIndicesUsed = currentIndicesMatches[0]  // listOfIndicesUsed means list of index that match
  const listOfIndexFocus = currentIndicesMatches[1]  // listOfIndexFocus means array of match and how many text it takes in text
  let indexAt = initialList(modifiedText.length, 0)
  if (currentIndex === 0) {
    return displayTextWithLinkWithHighlightWithFocus(text, listOfIndicesUsed, linkIndices, linkName, indexAt)
  }
  // when listOfIndexFocus.length !== 0
  orderChatIndex(listOfIndexFocus)
  const position = listOfIndexFocus[currentIndex - 1]
  // console.log('indexAt',indexAt)
  indexAt.fill(1, position[0], position[0] + position[1])
  return displayTextWithLinkWithHighlightWithFocus(text, listOfIndicesUsed, linkIndices, linkName, indexAt)
}

const getIndicesOfMatchesContain = (oldIndicesMatch, keyword, str, indexSpot) => {
  const keywordLen = keyword.length
  const result = oldIndicesMatch
  let tempIndices = initialList(oldIndicesMatch.length, 0)
  let startIndex = 0
  let foundIndex = -1
  do {
    foundIndex = str.indexOf(keyword, startIndex)
    if (foundIndex > -1) {
      result.fill(1, foundIndex, foundIndex + keywordLen)
      tempIndices.fill(1, foundIndex, foundIndex + keywordLen)
      indexSpot.push([foundIndex, keywordLen])
      startIndex = foundIndex + 1
    }
  } while (foundIndex > -1)
  return [result, indexSpot]
}

HighlightContain.propTypes = {
  text: PropTypes.string,
  keyword: PropTypes.string,
  currentIndex: PropTypes.number,
}

HighlightContain.defaultProps = {
  text: '',
  keyword: '',
  currentIndex: 1,
}

export default HighlightContain
