import React from 'react'
import PropTypes from 'prop-types'
import { displayTextWithLink, displayTextWithLinkWithHighlightWithFocus } from '../utils'
import { detectPartOfLink, getIndicesOfLink, getLinkUrl, orderChatIndex, initialList } from '../utils/text'
import '../index.scss'

const HighlightExact = ({ text, keywordExact, currentIndex }) => {
  const hasPartOfLink = detectPartOfLink(text)
  let linkIndices = initialList(text.length, 0)
  let linkName = initialList(text.length, 0)
  if (hasPartOfLink) {
    linkIndices = getIndicesOfLink(text, linkIndices)
    linkName = getLinkUrl(text, linkIndices)
  }
  // start display along as match and index logic
  const isKeywordHasAllSpace = keywordExact.trim()
  if (isKeywordHasAllSpace.length === 0) {
    if (!hasPartOfLink) {
      return (<span className="text-highlight-no-modify">{text}</span>)
    } else if (hasPartOfLink) {
      return displayTextWithLink(text, linkIndices, linkName)
    }
  }
  // when has keyword
  const modifiedText = text.toLowerCase()
  const trimedKeywordExact = keywordExact.toLowerCase().trim()
  // start logic match with index
  const listOfIndicesUseInText = getIndicesOfMatchesExact(modifiedText, trimedKeywordExact)
  let indexAt = initialList(modifiedText.length, 0)
  if (currentIndex === 0) {
    return displayTextWithLinkWithHighlightWithFocus(text, listOfIndicesUseInText[0], linkIndices, linkName, indexAt)
  }
  const listOfIndicesUsed = listOfIndicesUseInText[0]
  const listOfIndexFocus = listOfIndicesUseInText[1]
  orderChatIndex(listOfIndexFocus)
  // when listOfIndexFocus.length !== 0
  const position = listOfIndexFocus[currentIndex - 1]
  indexAt.fill(1, position[0], position[0] + position[1])
  return displayTextWithLinkWithHighlightWithFocus(text, listOfIndicesUsed, linkIndices, linkName, indexAt)
}

const getIndicesOfMatchesExact = (text, word) => {
  const textLength = text.length
  const wordLength = word.length
  let indexSpot = []
  let resultListWithIndices = initialList(textLength, 0)
  let startIndex = 0
  let index = -1
  const spaces = [' ', '\n', '\t']
  if (text === word) {
    resultListWithIndices.fill(1)
    indexSpot.push([0, wordLength])
  } else {
    index = text.indexOf(word, startIndex)
    while (index > -1) {
      if ((index === 0) && (spaces.includes(text[index + wordLength]))) {  // start of content
        resultListWithIndices.fill(1, index, index + wordLength)
        indexSpot.push([index, wordLength])
      } else if ((spaces.includes(text[index - 1])) && (spaces.includes(text[index + wordLength]))) {  // middle of content
        resultListWithIndices.fill(1, index, index + wordLength)
        indexSpot.push([index, wordLength])
      } else if ((spaces.includes(text[index - 1])) && (index + wordLength === textLength)) { // last of content
        resultListWithIndices.fill(1, index, index + wordLength)
        indexSpot.push([index, wordLength])
      }
      startIndex = index + wordLength
      index = text.indexOf(word, startIndex)
    }
  }
  // collect [index that match, index when click next and how many it will take]
  return [resultListWithIndices, indexSpot]
}

HighlightExact.propTypes = {
  text: PropTypes.string,
  keywordExact: PropTypes.string,
  currentIndex: PropTypes.number,
}

HighlightExact.defaultProps = {
  text: '',
  keywordExact: '',
  currentIndex: 1,
}

export default HighlightExact
